@@ -20,10 +20,11 @@
 #![feature(type_ascription)]
 #![feature(integer_atomics)]
 #![feature(fnbox)]
-#![deny(missing_docs)]
 
 extern crate cargo;
 extern crate env_logger;
+#[macro_use]
+extern crate failure;
 extern crate languageserver_types as ls_types;
 #[macro_use]
 extern crate lazy_static;
@@ -36,6 +37,7 @@ extern crate rls_data as data;
 extern crate rls_rustc as rustc_shim;
 extern crate rls_span as span;
 extern crate rls_vfs as vfs;
+#[cfg(feature = "rustfmt")]
 extern crate rustfmt_nightly as rustfmt;
 extern crate serde;
 #[macro_use]
@@ -68,17 +70,20 @@ const COMPILER_TIMEOUT: u64 = 1500;
 #[cfg(test)]
 const COMPILER_TIMEOUT: u64 = 3_600_000;
 
-const CRATE_BLACKLIST: [&'static str; 10] = [
+const CRATE_BLACKLIST: [&'static str; 13] = [
     "libc",
     "typenum",
     "alloc",
     "idna",
     "openssl",
-    "libunicode_normalization",
+    "unicode_normalization",
     "serde",
     "serde_json",
-    "librustc_serialize",
-    "libunicode_segmentation",
+    "rustc_serialize",
+    "unicode_segmentation",
+    "cocoa",
+    "gleam",
+    "winapi",
 ];
 
 const RUSTC_SHIM_ENV_VAR_NAME: &'static str = "RLS_RUSTC_SHIM";
