@@ -23,6 +23,7 @@ use expr::{rewrite_pair, rewrite_tuple, rewrite_unary_prefix, wrap_args_with_par
 use items::{format_generics_item_list, generics_shape_from_config};
 use lists::{definitive_tactic, itemize_list, write_list, ListFormatting, ListTactic, Separator,
             SeparatorPlace, SeparatorTactic};
+use macros::{rewrite_macro, MacroPosition};
 use rewrite::{Rewrite, RewriteContext};
 use shape::Shape;
 use utils::{colon_spaces, extra_offset, first_line_width, format_abi, format_mutability,
@@ -380,7 +381,7 @@ where
         FunctionRetTy::Default(..) => String::new(),
     };
 
-    let extendable = (!list_str.contains('\n') || list_str.is_empty()) && !output.contains("\n");
+    let extendable = (!list_str.contains('\n') || list_str.is_empty()) && !output.contains('\n');
     let args = wrap_args_with_parens(
         context,
         &list_str,
@@ -412,7 +413,7 @@ impl Rewrite for ast::WherePredicate {
         // TODO: dead spans?
         let result = match *self {
             ast::WherePredicate::BoundPredicate(ast::WhereBoundPredicate {
-                ref bound_lifetimes,
+                ref bound_generic_params,
                 ref bounded_ty,
                 ref bounds,
                 ..
@@ -421,9 +422,17 @@ impl Rewrite for ast::WherePredicate {
 
                 let colon = type_bound_colon(context);
 
-                if !bound_lifetimes.is_empty() {
-                    let lifetime_str: String = bound_lifetimes
+                if bound_generic_params
+                    .iter()
+                    .filter(|p| p.is_lifetime_param())
+                    .count() > 0
+                {
+                    let lifetime_str: String = bound_generic_params
                         .iter()
+                        .filter_map(|p| match p {
+                            &ast::GenericParam::Lifetime(ref l) => Some(l),
+                            _ => None,
+                        })
                         .map(|lt| lt.rewrite(context, shape))
                         .collect::<Option<Vec<_>>>()?
                         .join(", ");
@@ -442,10 +451,7 @@ impl Rewrite for ast::WherePredicate {
                     {
                         format!(
                             "for< {} > {}{}{}",
-                            lifetime_str,
-                            type_str,
-                            colon,
-                            bounds_str
+                            lifetime_str, type_str, colon, bounds_str
                         )
                     } else {
                         format!("for<{}> {}{}{}", lifetime_str, type_str, colon, bounds_str)
@@ -592,9 +598,17 @@ impl Rewrite for ast::TyParam {
 
 impl Rewrite for ast::PolyTraitRef {
     fn rewrite(&self, context: &RewriteContext, shape: Shape) -> Option<String> {
-        if !self.bound_lifetimes.is_empty() {
-            let lifetime_str: String = self.bound_lifetimes
+        if self.bound_generic_params
+            .iter()
+            .filter(|p| p.is_lifetime_param())
+            .count() > 0
+        {
+            let lifetime_str: String = self.bound_generic_params
                 .iter()
+                .filter_map(|p| match p {
+                    &ast::GenericParam::Lifetime(ref l) => Some(l),
+                    _ => None,
+                })
                 .map(|lt| lt.rewrite(context, shape))
                 .collect::<Option<Vec<_>>>()?
                 .join(", ");
@@ -720,14 +734,18 @@ impl Rewrite for ast::Ty {
                     SeparatorPlace::Back,
                 )
             }
-            ast::TyKind::Infer => if shape.width >= 1 {
-                Some("_".to_owned())
-            } else {
-                None
-            },
+            ast::TyKind::Infer => {
+                if shape.width >= 1 {
+                    Some("_".to_owned())
+                } else {
+                    None
+                }
+            }
             ast::TyKind::BareFn(ref bare_fn) => rewrite_bare_fn(bare_fn, self.span, context, shape),
             ast::TyKind::Never => Some(String::from("!")),
-            ast::TyKind::Mac(..) => None,
+            ast::TyKind::Mac(ref mac) => {
+                rewrite_macro(mac, None, context, shape, MacroPosition::Expression)
+            }
             ast::TyKind::ImplicitSelf => Some(String::from("")),
             ast::TyKind::ImplTrait(ref it) => it.rewrite(context, shape)
                 .map(|it_str| format!("impl {}", it_str)),
@@ -744,14 +762,23 @@ fn rewrite_bare_fn(
 ) -> Option<String> {
     let mut result = String::with_capacity(128);
 
-    if !bare_fn.lifetimes.is_empty() {
+    if bare_fn
+        .generic_params
+        .iter()
+        .filter(|p| p.is_lifetime_param())
+        .count() > 0
+    {
         result.push_str("for<");
         // 6 = "for<> ".len(), 4 = "for<".
         // This doesn't work out so nicely for mutliline situation with lots of
         // rightward drift. If that is a problem, we could use the list stuff.
         result.push_str(&bare_fn
-            .lifetimes
+            .generic_params
             .iter()
+            .filter_map(|p| match p {
+                &ast::GenericParam::Lifetime(ref l) => Some(l),
+                _ => None,
+            })
             .map(|l| {
                 l.rewrite(
                     context,
